"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const docker_command_1 = __importDefault(require("../../docker-command"));
const docker_1 = require("../../utils/docker");
class Logs extends docker_command_1.default {
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { flags } = this.parse(Logs);
            const services = yield this.listServices({
                name: flags.name
            });
            if (services.length === 0) {
                throw new Error("No engine is running.");
            }
            const service = services[0];
            const logs = yield service.logs({
                stderr: true,
                stdout: true,
                follow: flags.follow,
                tail: flags.tail && flags.tail >= 0 ? flags.tail : 'all'
            });
            logs.on('data', (buffer) => docker_1.parseLog(buffer).forEach(x => this.log(x)));
            logs.on('error', (error) => {
                throw error;
            });
            return logs;
        });
    }
}
exports.default = Logs;
Logs.description = 'Show the Engine\'s logs';
Logs.flags = Object.assign(Object.assign({}, docker_command_1.default.flags), { tail: command_1.flags.integer({
        description: 'Display the last N lines',
        default: 10000
    }), follow: command_1.flags.boolean({
        description: 'Follow logs',
        allowNo: true,
        default: true
    }) });
