"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const docker_command_1 = __importDefault(require("../../docker-command"));
const status_1 = __importDefault(require("./status"));
class Stop extends docker_command_1.default {
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { flags } = this.parse(Stop);
            const status = yield status_1.default.run(['--name', flags.name, '--silent', ...this.flagsAsArgs(flags)]);
            if (status === 1 /* STOPPED */) {
                this.log('Engine is already stopped');
                return false;
            }
            this.spinner.start('Stopping Engine');
            const services = yield this.listServices({ name: flags.name });
            if (services.length === 0)
                return false;
            const service = services[0];
            const eventPromise = this.waitForEvent(({ Action, from, Type }) => Type === 'container' &&
                Action === 'destroy' &&
                from === service.data.Spec.TaskTemplate.ContainerSpec.Image);
            yield service.remove();
            yield eventPromise;
            this.spinner.stop();
            return true;
        });
    }
}
exports.default = Stop;
Stop.description = 'Stop the Engine';
Stop.flags = Object.assign({}, docker_command_1.default.flags);
