"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const base58 = __importStar(require("@mesg/api/lib/util/base58"));
const root_command_1 = __importDefault(require("../../root-command"));
const error_1 = require("../../utils/error");
const compile_1 = __importDefault(require("./compile"));
const create_1 = __importDefault(require("./create"));
const delete_1 = __importDefault(require("./delete"));
const logs_1 = __importDefault(require("./logs"));
class ProcessDev extends root_command_1.default {
    constructor() {
        super(...arguments);
        this.processCreated = false;
        this.instanceCreated = false;
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { args, flags } = this.parse(ProcessDev);
            this.spinner.start('Starting process');
            this.spinner.status = 'compiling';
            const envToArgs = (flags.env || []).reduce((prev, next) => [...prev, '--env', next], []);
            const definition = yield compile_1.default.run([args.PROCESS, '--silent', '--dev', ...envToArgs, ...this.flagsAsArgs(flags)]);
            this.spinner.status = 'creating process';
            const processHash = yield this.createProcess(definition);
            this.spinner.status = 'fetching logs';
            const stream = yield logs_1.default.run([base58.encode(processHash), ...this.flagsAsArgs(flags)]);
            this.spinner.stop('ready');
            process.once('SIGINT', () => __awaiter(this, void 0, void 0, function* () {
                stream.destroy();
                if (this.processCreated)
                    yield delete_1.default.run([base58.encode(processHash), '--confirm', ...this.flagsAsArgs(flags)]);
            }));
        });
    }
    createProcess(definition) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const process = yield this.api.process.create(definition);
                if (!process.hash)
                    throw new Error('invalid hash');
                this.processCreated = true;
                return process.hash;
            }
            catch (e) {
                if (!error_1.IsAlreadyExistsError.match(e))
                    throw e;
                this.warn('process already created');
                return new error_1.IsAlreadyExistsError(e).hash;
            }
        });
    }
}
exports.default = ProcessDev;
ProcessDev.description = 'Run a process in development mode';
ProcessDev.flags = Object.assign(Object.assign(Object.assign({}, root_command_1.default.flags), compile_1.default.flags), create_1.default.flags);
ProcessDev.args = [{
        name: 'PROCESS',
        description: 'Path of the process',
        default: './'
    }];
