"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const base58 = __importStar(require("@mesg/api/lib/util/base58"));
const root_command_1 = __importDefault(require("../../root-command"));
const error_1 = require("../../utils/error");
const compile_1 = __importDefault(require("./compile"));
const create_1 = __importDefault(require("./create"));
const logs_1 = __importDefault(require("./logs"));
const start_1 = __importDefault(require("./start"));
const stop_1 = __importDefault(require("./stop"));
class ServiceDev extends root_command_1.default {
    constructor() {
        super(...arguments);
        this.runnerCreated = false;
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { args, flags } = this.parse(ServiceDev);
            this.spinner.start('Starting service');
            this.spinner.status = 'compiling';
            const definition = yield compile_1.default.run([args.SERVICE, '--silent', ...this.flagsAsArgs(flags)]);
            this.spinner.status = 'creating';
            const serviceHash = yield this.createService(definition);
            this.spinner.status = 'starting';
            const runnerHash = yield this.startService(serviceHash, flags.env);
            this.spinner.status = 'fetching logs';
            const stream = yield logs_1.default.run([base58.encode(runnerHash), ...this.flagsAsArgs(flags)]);
            this.spinner.stop(base58.encode(runnerHash));
            process.once('SIGINT', () => __awaiter(this, void 0, void 0, function* () {
                stream.destroy();
                if (this.runnerCreated)
                    yield stop_1.default.run([base58.encode(runnerHash), ...this.flagsAsArgs(flags)]);
            }));
        });
    }
    createService(definition) {
        return __awaiter(this, void 0, void 0, function* () {
            const { hash } = yield this.api.service.hash(definition);
            if (!hash)
                throw new Error('invalid hash');
            const { exists } = yield this.api.service.exists({ hash });
            if (!exists) {
                const service = yield this.api.service.create(definition);
                if (!service.hash)
                    throw new Error('invalid hash');
                if (service.hash.toString() !== hash.toString())
                    throw new Error('invalid hash');
            }
            return hash;
        });
    }
    startService(serviceHash, env) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const { hash } = yield this.api.runner.create({
                    serviceHash,
                    env
                });
                if (!hash)
                    throw new Error('invalid hash');
                this.runnerCreated = true;
                return hash;
            }
            catch (e) {
                if (!error_1.IsAlreadyExistsError.match(e))
                    throw e;
                this.warn('service already started');
                return new error_1.IsAlreadyExistsError(e).hash;
            }
        });
    }
}
exports.default = ServiceDev;
ServiceDev.description = 'Run a service in development mode';
ServiceDev.flags = Object.assign(Object.assign(Object.assign({}, root_command_1.default.flags), create_1.default.flags), start_1.default.flags);
ServiceDev.args = [{
        name: 'SERVICE',
        description: 'Path or url of a service',
        default: './'
    }];
