"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const fs_1 = require("fs");
const base58 = __importStar(require("@mesg/api/lib/util/base58"));
const root_command_1 = __importDefault(require("../../root-command"));
const resolver_1 = require("../../utils/resolver");
class ServiceExecute extends root_command_1.default {
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { args, flags } = this.parse(ServiceExecute);
            const runnerHash = yield resolver_1.runnerResolver(this.api, args.RUNNER_HASH);
            const runner = yield this.api.runner.get({
                hash: runnerHash
            });
            if (!runner.instanceHash) {
                throw new Error('invalid runner hash');
            }
            const instance = yield this.api.instance.get({
                hash: runner.instanceHash
            });
            if (!instance.serviceHash) {
                throw new Error('invalid instance');
            }
            const service = yield this.api.service.get({
                hash: instance.serviceHash
            });
            if (!service.hash) {
                throw new Error('invalid service');
            }
            const task = service.tasks.find(x => x.key === args.TASK);
            if (!task) {
                throw new Error(`The task ${args.TASK} does not exist in service '${base58.encode(service.hash)}'`);
            }
            const result = yield this.execute({
                inputs: this.convertValue(task.inputs, this.dataFromFlags(flags)),
                executorHash: runnerHash,
                tags: ['CLI'],
                taskKey: args.TASK,
                eventHash: base58.decode(flags.eventHash || '6aUPZhmnFKiSsHXRaddbnqsKKi9KogbQNiKUcpivaohb'),
            });
            this.styledJSON(result);
            return result;
        });
    }
    dataFromFlags(flags) {
        if (flags.json) {
            return JSON.parse(fs_1.readFileSync(flags.json).toString());
        }
        return (flags.data || []).reduce((acc, item) => {
            const [key, value] = item.split('=');
            return Object.assign(Object.assign({}, acc), { [key]: value });
        }, {});
    }
    convertValue(inputs, data) {
        if (!inputs)
            return {};
        return inputs
            .filter((x) => data[x.key] !== undefined && data[x.key] !== null)
            .reduce((prev, value) => (Object.assign(Object.assign({}, prev), { [value.key]: this.convert(value.type, data[value.key]) })), {});
    }
    convert(type, value) {
        try {
            return {
                Object: (x) => typeof x === 'string' ? JSON.parse(x) : x,
                String: (x) => x,
                Boolean: (x) => ['true', 't', 'TRUE', 'T', '1'].includes(x),
                Number: (x) => parseFloat(x),
                Any: (x) => x,
            }[type](value);
        }
        catch (_a) {
            this.warn(`Cannot parse ${value} in ${type}`);
            return value;
        }
    }
}
exports.default = ServiceExecute;
ServiceExecute.description = 'Execute a task on a running service';
ServiceExecute.flags = Object.assign(Object.assign({}, root_command_1.default.flags), { json: command_1.flags.string({ char: 'j', description: 'Path to a JSON file containing the task inputs' }), data: command_1.flags.string({
        char: 'd',
        description: 'Task inputs',
        multiple: true,
        helpValue: 'key=value'
    }), eventHash: command_1.flags.string({
        description: 'Event hash to create the execution with'
    }) });
ServiceExecute.args = [{
        name: 'RUNNER_HASH',
        required: true,
        description: 'The hash of the runner that will execute this execution'
    }, {
        name: 'TASK',
        required: true,
        description: 'Task key'
    }];
