"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const chalk_1 = __importDefault(require("chalk"));
const types_1 = require("@mesg/api/lib/types");
const base58 = __importStar(require("@mesg/api/lib/util/base58"));
const encoder_1 = require("@mesg/api/lib/util/encoder");
const node_docker_api_1 = require("node-docker-api");
const root_command_1 = __importDefault(require("../../root-command"));
const docker_1 = require("../../utils/docker");
const resolver_1 = require("../../utils/resolver");
class ServiceLogs extends root_command_1.default {
    constructor() {
        super(...arguments);
        this.docker = new node_docker_api_1.Docker(null);
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { args, flags } = this.parse(ServiceLogs);
            const runnerHash = yield resolver_1.runnerResolver(this.api, args.RUNNER_HASH);
            const runner = yield this.api.runner.get({ hash: runnerHash });
            if (!runner.hash) {
                throw new Error('runner is invalid');
            }
            const streams = [];
            const dockerServices = yield this.docker.service.list({
                filters: {
                    label: [`mesg.runner=${base58.encode(runnerHash)}`]
                }
            });
            for (const service of dockerServices) {
                const labels = service.data.Spec.Labels;
                const name = [labels['mesg.sid'], labels['mesg.dependency']].join('/');
                const logs = yield service.logs({
                    stderr: true,
                    stdout: true,
                    follow: flags.follow,
                    tail: flags.tail && flags.tail >= 0 ? flags.tail : 'all'
                });
                streams.push(() => logs.destroy());
                logs
                    .on('data', (buffer) => docker_1.parseLog(buffer).forEach(x => this.log(chalk_1.default.gray(name + ':'), x)))
                    .on('error', (error) => { this.warn('Docker log stream error: ' + error.message); });
            }
            if (flags.results) {
                const results = this.api.execution.stream({
                    filter: {
                        executorHash: runnerHash,
                        statuses: [
                            types_1.ExecutionStatus.COMPLETED,
                            types_1.ExecutionStatus.FAILED,
                        ],
                        taskKey: flags.task
                    }
                });
                streams.push(() => results.cancel());
                results
                    .on('data', data => this.log(this.formatResult(data)))
                    .on('error', (error) => { this.warn('Result stream error: ' + error.message); });
            }
            if (flags.events) {
                const events = this.api.event.stream({
                    filter: {
                        instanceHash: runner.instanceHash,
                        key: flags.event,
                    }
                });
                streams.push(() => events.cancel());
                events
                    .on('data', (data) => this.log(this.formatEvent(data)))
                    .on('error', (error) => { this.warn('Event stream error: ' + error.message); });
            }
            return {
                destroy: () => {
                    streams.forEach(s => s());
                }
            };
        });
    }
    formatEvent(event) {
        if (!event.data)
            return;
        return `EVENT[${event.key}]: ` + chalk_1.default.gray(JSON.stringify(encoder_1.decode(event.data)));
    }
    formatResult(execution) {
        if (execution.error) {
            return `RESULT[${execution.taskKey}]: ` + chalk_1.default.red('ERROR:', execution.error);
        }
        if (!execution.outputs)
            return;
        return `RESULT[${execution.taskKey}]: ` + chalk_1.default.gray(JSON.stringify(encoder_1.decode(execution.outputs)));
    }
}
exports.default = ServiceLogs;
ServiceLogs.description = 'Fetch the logs of a service';
ServiceLogs.flags = Object.assign(Object.assign({}, root_command_1.default.flags), { 
    // dependency: flags.string({
    //   char: 'd',
    //   description: 'Name of the dependency to show the logs from',
    //   multiple: true
    // }),
    events: command_1.flags.boolean({
        description: 'Display events',
        allowNo: true,
        default: true
    }), results: command_1.flags.boolean({
        description: 'Display results',
        allowNo: true,
        default: true
    }), event: command_1.flags.string({
        description: 'Display a specific event'
    }), task: command_1.flags.string({
        description: 'Display a specific task results'
    }), tail: command_1.flags.integer({
        description: 'Display the last N lines',
        default: 10000
    }), follow: command_1.flags.boolean({
        description: 'Follow log output',
        allowNo: true,
        default: true
    }) });
ServiceLogs.args = [{
        name: 'RUNNER_HASH',
        required: true,
    }];
