"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const cli_ux_1 = __importDefault(require("cli-ux"));
const root_command_1 = __importDefault(require("../../root-command"));
const resolver_1 = require("../../utils/resolver");
class ServiceStop extends root_command_1.default {
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { argv, flags } = this.parse(ServiceStop);
            if (flags['delete-data'] && !flags.confirm) {
                cli_ux_1.default.warn('This will delete all the data associated with this running service');
                if (!(yield cli_ux_1.default.confirm('Are you sure?')))
                    return [];
            }
            this.spinner.start('Stop running services');
            for (const hash of argv) {
                const runnerHash = yield resolver_1.runnerResolver(this.api, hash);
                yield this.api.runner.delete({ hash: runnerHash, deleteData: flags['delete-data'] });
            }
            this.spinner.stop(argv.join(', '));
            return argv;
        });
    }
}
exports.default = ServiceStop;
ServiceStop.description = 'Stop one or more running service';
ServiceStop.flags = Object.assign(Object.assign({}, root_command_1.default.flags), { 'delete-data': command_1.flags.boolean({
        description: 'Delete running service persistent data',
        default: false,
    }), confirm: command_1.flags.boolean({
        description: 'Confirm deletion',
        default: false,
    }) });
ServiceStop.strict = false;
ServiceStop.args = [{
        name: 'RUNNER_HASH...',
        required: true,
    }];
