"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const cli_ux_1 = require("cli-ux");
const application_1 = __importDefault(require("@mesg/application"));
const api_1 = __importDefault(require("@mesg/api"));
const base58 = __importStar(require("@mesg/api/lib/util/base58"));
const util_1 = require("util");
class default_1 extends command_1.Command {
    constructor(argv, config) {
        super(argv, config);
        const { flags } = this.parse();
        const port = flags.port;
        const host = process.env.DOCKER_HOST
            ? new URL(process.env.DOCKER_HOST).hostname
            : flags.host;
        const endpoint = `${host}:${port}`;
        this.api = new api_1.default(endpoint);
        this._app = new application_1.default(this.api);
    }
    get spinner() {
        const { flags } = this.parse();
        const nope = () => { };
        if (flags.quiet) {
            return { start: nope, stop: (message) => message ? this.log(message) : null, status: null };
        }
        if (flags.silent) {
            return { start: nope, stop: nope, status: null };
        }
        return cli_ux_1.cli.action;
    }
    flagsAsArgs({ port, host }) {
        return ['--port', port.toString(), '--host', host.toString()];
    }
    log(message, ...args) {
        if (this.parse) {
            const { flags } = this.parse();
            if (flags.silent) {
                return;
            }
        }
        message = typeof message === 'string' ? message : util_1.inspect(message);
        process.stdout.write(util_1.format(message, ...args) + '\n');
    }
    styledJSON(data) {
        const { flags } = this.parse();
        if (flags.silent)
            return;
        const base58EncodedHash = JSON.parse(JSON.stringify(data, (key, value) => {
            return key && RegExp('hash$', 'i').test(key) && value && value.type === 'Buffer'
                ? base58.encode(value.data)
                : value;
        }));
        cli_ux_1.cli.styledJSON(base58EncodedHash);
    }
    catch(err) {
        const _super = Object.create(null, {
            catch: { get: () => super.catch }
        });
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield _super.catch.call(this, err);
            }
            catch (e) {
                if (e.message) {
                    this.error(err.message);
                }
                else {
                    throw e;
                }
            }
        });
    }
    execute(request) {
        return __awaiter(this, void 0, void 0, function* () {
            const exec = yield this._app.executeTaskAndWaitResult({
                executorHash: request.executorHash,
                tags: request.tags || [],
                taskKey: request.taskKey,
                inputs: this._app.encodeData(request.inputs || {}),
                eventHash: request.eventHash,
            });
            if (exec.error)
                throw new Error(exec.error);
            if (!exec.outputs)
                throw new Error('missing outputs');
            return this._app.decodeData(exec.outputs);
        });
    }
}
exports.default = default_1;
default_1.flags = {
    help: command_1.flags.help({ char: 'h' }),
    quiet: command_1.flags.boolean({ char: 'q', description: 'Display only essential information' }),
    silent: command_1.flags.boolean({ hidden: true }),
    port: command_1.flags.integer({ char: 'p', default: 50052, description: 'Port to access the MESG engine' }),
    host: command_1.flags.string({ default: 'localhost', description: 'Host to access the MESG engine' })
};
